@@ -1,15 +1,29 @@
 #
-# Copyright 2009 Optaros, Inc.
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
+#
+#   Authors:
+#    Jeff Potts, Optaros
 #
 '''
 Unit tests for cmislib
 '''
 import unittest
-from cmislib.model import CmisClient
+from cmislib.model import CmisClient, ACE
 from cmislib.exceptions import \
                           ObjectNotFoundException, \
                           PermissionDeniedException, \
                           CmisException
+from cmislib import messages
 import os
 from time import sleep, time
 import settings
@@ -452,6 +466,65 @@ class FolderTest(CmisTestBase):
         self.assert_('cmis:name' in props)
         self.assert_(props['cmis:name'] != None)
 
+    def testPropertyFilter(self):
+        '''Test the properties filter'''
+        # names of folders and test docs
+        testFolderName = self._testFolder.getName()
+        parentFolderName = 'testGetObjectByPath folder'
+        subFolderName = 'subfolder'
+
+        # create the folder structure
+        parentFolder = self._testFolder.createFolder(parentFolderName)
+        subFolder = parentFolder.createFolder(subFolderName)
+
+        # Per CMIS-170, CMIS providers are not required to filter the
+        # properties returned. So these tests will check only for the presence
+        # of the properties asked for, not the absence of properties that
+        # should be filtered if the server chooses to do so.
+
+        # test when used with getObjectByPath
+        searchFolder = self._repo.getObjectByPath(settings.TEST_ROOT_PATH + \
+                        "/".join([testFolderName, parentFolderName, subFolderName]), \
+                        filter='cmis:objectId,cmis:objectTypeId,cmis:baseTypeId')
+        self.assertEquals(subFolder.getObjectId(), searchFolder.getObjectId())
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectTypeId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:baseTypeId'))
+
+        # test when used with getObjectByPath + reload
+        searchFolder = self._repo.getObjectByPath(settings.TEST_ROOT_PATH + \
+                        "/".join([testFolderName, parentFolderName, subFolderName]), \
+                        filter='cmis:objectId,cmis:objectTypeId,cmis:baseTypeId')
+        searchFolder.reload()
+        self.assertEquals(subFolder.getObjectId(), searchFolder.getObjectId())
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectTypeId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:baseTypeId'))
+
+        # test when used with getObject
+        searchFolder = self._repo.getObject(subFolder.getObjectId(), \
+                        filter='cmis:objectId,cmis:objectTypeId,cmis:baseTypeId')
+        self.assertEquals(subFolder.getObjectId(), searchFolder.getObjectId())
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectTypeId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:baseTypeId'))
+
+        # test when used with getObject + reload
+        searchFolder = self._repo.getObject(subFolder.getObjectId(), \
+                        filter='cmis:objectId,cmis:objectTypeId,cmis:baseTypeId')
+        searchFolder.reload()
+        self.assertEquals(subFolder.getObjectId(), searchFolder.getObjectId())
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectTypeId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:baseTypeId'))
+
+        # test that you can do a reload with a reset filter
+        searchFolder.reload(filter='*')
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:objectTypeId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:baseTypeId'))
+        self.assertTrue(searchFolder.getProperties().has_key('cmis:name'))
+
     def testUpdateProperties(self):
         '''Create a test folder, then update its properties'''
         folderName = 'testUpdateProperties folder'
@@ -496,16 +569,99 @@ class FolderTest(CmisTestBase):
                           firstFolder.createFolder,
                           'bad parent')
 
-    def testDuplicateFolder(self):
-        '''Try to create a folder that already exists'''
-        folderName = 'testDupFolder folder'
-        firstFolder = self._testFolder.createFolder(folderName)
-        self.assert_('cmis:objectId' in firstFolder.getProperties())
-        # really, this needs to be ContentAlreadyExistsException but
-        # not all CMIS providers report it as such
-        self.assertRaises(CmisException,
-                          self._testFolder.createFolder,
-                          folderName)
+# Per CMIS-169, nothing in the spec says that an exception should be thrown
+# when a duplicate folder is created, so this test is really not necessary.
+#    def testDuplicateFolder(self):
+#        '''Try to create a folder that already exists'''
+#        folderName = 'testDupFolder folder'
+#        firstFolder = self._testFolder.createFolder(folderName)
+#        self.assert_('cmis:objectId' in firstFolder.getProperties())
+#        # really, this needs to be ContentAlreadyExistsException but
+#        # not all CMIS providers report it as such
+#        self.assertRaises(CmisException,
+#                          self._testFolder.createFolder,
+#                          folderName)
+
+
+class ChangeEntryTest(CmisTestBase):
+
+    """ Tests for the :class:`ChangeEntry` class """
+
+    def testGetContentChanges(self):
+
+        """Get the content changes and inspect Change Entry props"""
+
+        # need to check changes capability
+        if not self._repo.capabilities['Changes']:
+            print messages.NO_CHANGE_LOG_SUPPORT
+            return
+
+        # at least one change should have been made due to the creation of the
+        # test documents
+        rs = self._repo.getContentChanges()
+        self.assertTrue(len(rs) > 0)
+        changeEntry = rs[0]
+        self.assertTrue(changeEntry.id)
+        self.assertTrue(changeEntry.changeType in ['created', 'updated', 'deleted'])
+        self.assertTrue(changeEntry.changeTime)
+
+    def testGetACL(self):
+
+        """Gets the ACL that is included with a Change Entry."""
+
+        # need to check changes capability
+        if not self._repo.capabilities['Changes']:
+            print messages.NO_CHANGE_LOG_SUPPORT
+            return
+
+        # need to check ACL capability
+        if not self._repo.capabilities['ACL']:
+            print messages.NO_ACL_SUPPORT
+            return
+
+        # need to test once with includeACL set to true
+        rs = self._repo.getContentChanges(includeACL='true')
+        self.assertTrue(len(rs) > 0)
+        changeEntry = rs[0]
+        acl = changeEntry.getACL()
+        self.assertTrue(acl)
+        for entry in acl.getEntries().values():
+            self.assertTrue(entry.principalId)
+            self.assertTrue(entry.permissions)
+
+        # need to test once without includeACL set
+        rs = self._repo.getContentChanges()
+        self.assertTrue(len(rs) > 0)
+        changeEntry = rs[0]
+        acl = changeEntry.getACL()
+        self.assertTrue(acl)
+        for entry in acl.getEntries().values():
+            self.assertTrue(entry.principalId)
+            self.assertTrue(entry.permissions)
+
+    def testGetProperties(self):
+
+        """Gets the properties of an object included with a Change Entry."""
+
+        # need to check changes capability
+        changeCap = self._repo.capabilities['Changes']
+        if not changeCap:
+            print messages.NO_CHANGE_LOG_SUPPORT
+            return
+
+        # need to test once without includeProperties set. the objectID should be there
+        rs = self._repo.getContentChanges()
+        self.assertTrue(len(rs) > 0)
+        changeEntry = rs[0]
+        self.assertTrue(changeEntry.properties['cmis:objectId'])
+
+        # need to test once with includeProperties set. the objectID should be there plus object props
+        if changeCap in ['properties', 'all']:
+            rs = self._repo.getContentChanges(includeProperties='true')
+            self.assertTrue(len(rs) > 0)
+            changeEntry = rs[0]
+            self.assertTrue(changeEntry.properties['cmis:objectId'])
+            self.assertTrue(changeEntry.properties['cmis:name'])
 
 
 class DocumentTest(CmisTestBase):
@@ -538,10 +694,27 @@ class DocumentTest(CmisTestBase):
             self.assertTrue(testDoc.isCheckedOut())
             self.assert_('cmis:objectId' in testDoc.getProperties())
             self.assert_('cmis:objectId' in pwcDoc.getProperties())
+            testDoc = pwcDoc.checkin()
+            self.assertFalse(testDoc.isCheckedOut())
+        finally:
+            if testDoc.isCheckedOut():
+                pwcDoc.delete()
+
+    def testCheckinComment(self):
+        '''Checkin a document with a comment'''
+        testFilename = settings.TEST_BINARY_1
+        contentFile = open(testFilename, 'rb')
+        testDoc = self._testFolder.createDocument(testFilename, contentFile=contentFile)
+        contentFile.close()
+        self.assertEquals(testFilename, testDoc.getName())
+        pwcDoc = testDoc.checkout()
+
+        try:
+            self.assertTrue(testDoc.isCheckedOut())
             testDoc = pwcDoc.checkin(checkinComment='Just a few changes')
+            self.assertFalse(testDoc.isCheckedOut())
             self.assertEquals('Just a few changes',
                           testDoc.getProperties()['cmis:checkinComment'])
-            self.assertFalse(testDoc.isCheckedOut())
         finally:
             if testDoc.isCheckedOut():
                 pwcDoc.delete()
@@ -561,10 +734,8 @@ class DocumentTest(CmisTestBase):
             self.assertTrue(testDoc.isCheckedOut())
             self.assert_('cmis:objectId' in testDoc.getProperties())
             self.assert_('cmis:objectId' in pwcDoc.getProperties())
-            testDoc = pwcDoc.checkin(checkinComment='Just a few changes')
+            testDoc = pwcDoc.checkin()
             self.assertFalse(testDoc.isCheckedOut())
-            self.assertEquals('Just a few changes',
-                          testDoc.getProperties()['cmis:checkinComment'])
         finally:
             if testDoc.isCheckedOut():
                 pwcDoc.delete()
@@ -769,7 +940,7 @@ class DocumentTest(CmisTestBase):
         pwc.delete()
 
     def testCreateDocumentBinary(self):
-        ''' Create a document using a file from the file system '''
+        '''Create a binary document using a file from the file system'''
         testFilename = settings.TEST_BINARY_1
         contentFile = open(testFilename, 'rb')
         newDoc = self._testFolder.createDocument(testFilename, contentFile=contentFile)
@@ -791,7 +962,7 @@ class DocumentTest(CmisTestBase):
         os.remove(exportFilename)
 
     def testCreateDocumentPlain(self):
-        ''' Create a document using a file from the file system '''
+        '''Create a plain document using a file from the file system'''
         testFilename = 'plain.txt'
         testFile = open(testFilename, 'w')
         testFile.write('This is a sample text file line 1.\n')
@@ -850,12 +1021,13 @@ class TypeTest(unittest.TestCase):
         cmisClient = CmisClient(settings.REPOSITORY_URL, settings.USERNAME, settings.PASSWORD)
         repo = cmisClient.getDefaultRepository()
         typeDefs = repo.getTypeDescendants()
-        found = False
+        folderDef = None
         for typeDef in typeDefs:
             if typeDef.getTypeId() == 'cmis:folder':
-                found = True
+                folderDef = typeDef
                 break
-        self.assertTrue(found)
+        self.assertTrue(folderDef)
+        self.assertTrue(folderDef.baseId)
 
     def testTypeChildren(self):
         '''Get the child types for this repository and make sure cmis:folder
@@ -867,12 +1039,13 @@ class TypeTest(unittest.TestCase):
         cmisClient = CmisClient(settings.REPOSITORY_URL, settings.USERNAME, settings.PASSWORD)
         repo = cmisClient.getDefaultRepository()
         typeDefs = repo.getTypeChildren()
-        found = False
+        folderDef = None
         for typeDef in typeDefs:
             if typeDef.getTypeId() == 'cmis:folder':
-                found = True
+                folderDef = typeDef
                 break
-        self.assertTrue(found)
+        self.assertTrue(folderDef)
+        self.assertTrue(folderDef.baseId)
 
     def testTypeDefinition(self):
         '''Get the cmis:document type and test a few props of the type.'''
@@ -880,7 +1053,87 @@ class TypeTest(unittest.TestCase):
         repo = cmisClient.getDefaultRepository()
         docTypeDef = repo.getTypeDefinition('cmis:document')
         self.assertEquals('cmis:document', docTypeDef.getTypeId())
-        # Add some property inspection stuff here
+        self.assertTrue(docTypeDef.baseId)
+
+    def testTypeProperties(self):
+        '''Get the properties for a type.'''
+        cmisClient = CmisClient(settings.REPOSITORY_URL, settings.USERNAME, settings.PASSWORD)
+        repo = cmisClient.getDefaultRepository()
+        docTypeDef = repo.getTypeDefinition('cmis:document')
+        self.assertEquals('cmis:document', docTypeDef.getTypeId())
+        props = docTypeDef.getProperties().values()
+        self.assertTrue(len(props) > 0)
+        for prop in props:
+            if prop.queryable:
+                self.assertTrue(prop.queryName)
+            self.assertTrue(prop.propertyType)
+
+
+class ACLTest(CmisTestBase):
+
+    """
+    Tests related to :class:`ACL` and :class:`ACE`
+    """
+
+    def testSupportedPermissions(self):
+        '''Test the value of supported permissions enum'''
+        if not self._repo.getCapabilities()['ACL']:
+            print messages.NO_ACL_SUPPORT
+            return
+        self.assertTrue(self._repo.getSupportedPermissions() in ['basic', 'repository', 'both'])
+
+    def testPermissionDefinitions(self):
+        '''Test the list of permission definitions'''
+        if not self._repo.getCapabilities()['ACL']:
+            print messages.NO_ACL_SUPPORT
+            return
+        supportedPerms = self._repo.getPermissionDefinitions()
+        self.assertTrue(supportedPerms.has_key('cmis:write'))
+
+    def testPermissionMap(self):
+        '''Test the permission mapping'''
+        if not self._repo.getCapabilities()['ACL']:
+            print messages.NO_ACL_SUPPORT
+            return
+        permMap = self._repo.getPermissionMap()
+        self.assertTrue(permMap.has_key('canGetProperties.Object'))
+        self.assertTrue(len(permMap['canGetProperties.Object']) > 0)
+
+    def testPropagation(self):
+        '''Test the propagation setting'''
+        if not self._repo.getCapabilities()['ACL']:
+            print messages.NO_ACL_SUPPORT
+            return
+        self.assertTrue(self._repo.getPropagation() in ['objectonly', 'propagate', 'repositorydetermined'])
+
+    def testGetObjectACL(self):
+        '''Test getting an object's ACL'''
+        if not self._repo.getCapabilities()['ACL']:
+            print messages.NO_ACL_SUPPORT
+            return
+        acl = self._testFolder.getACL()
+        for entry in acl.getEntries().values():
+            self.assertTrue(entry.principalId)
+            self.assertTrue(entry.permissions)
+
+    def testApplyACL(self):
+        '''Test updating an object's ACL'''
+        if not self._repo.getCapabilities()['ACL']:
+            print messages.NO_ACL_SUPPORT
+            return
+        if not self._repo.getCapabilities()['ACL'] == 'manage':
+            print 'Repository does not support manage ACL'
+            return
+        if not self._repo.getSupportedPermissions() in ['both', 'basic']:
+            print 'Repository needs to support either both or basic permissions for this test'
+            return
+        acl = self._testFolder.getACL()
+        acl.addEntry(ACE('jpotts', 'cmis:write', 'true'))
+        acl = self._testFolder.applyACL(acl)
+        # would be good to check that the permission we get back is what we set
+        # but at least one server (Alf) appears to map the basic perm to a
+        # repository-specific perm
+        self.assertTrue(acl.getEntries().has_key('jpotts'))
 
 
 def isInCollection(collection, targetDoc):
@@ -909,7 +1162,7 @@ def isInResultSet(resultSet, targetDoc):
     """
     done = False
     while not done:
-        if resultSet.getResults().has_key(targetDoc.getObjectId()):
+        if resultSet.hasObject(targetDoc.getObjectId()):
             return True
         if resultSet.hasNext():
             resultSet.getNext()
