@@ -1,5 +1,6 @@
-from unittest import TestSuite, TestResult, TestLoader
-from tests.cmislibtest import CmisClientTest, QueryTest, FolderTest, DocumentTest, TypeTest, RepositoryTest
+from unittest import TestSuite, TestResult, TestLoader, TextTestRunner
+from tests.cmislibtest import CmisClientTest, QueryTest, FolderTest, \
+        DocumentTest, TypeTest, RepositoryTest, ACLTest, ChangeEntryTest
 
 # Nuxeo working tests
 # - CmisClientTest
@@ -27,12 +28,15 @@ from tests.cmislibtest import CmisClientTest, QueryTest, FolderTest, DocumentTes
 
 tts = TestSuite()
 
-tts = TestLoader().loadTestsFromTestCase(CmisClientTest)
-tts = TestLoader().loadTestsFromTestCase(RepositoryTest)
-tts = TestLoader().loadTestsFromTestCase(DocumentTest)
+#tts.addTests(TestLoader().loadTestsFromTestCase(CmisClientTest))
+#tts.addTests(TestLoader().loadTestsFromTestCase(RepositoryTest))
+#tts.addTests(TestLoader().loadTestsFromTestCase(FolderTest))
+#tts.addTests(TestLoader().loadTestsFromTestCase(DocumentTest))
+#tts.addTests(TestLoader().loadTestsFromTestCase(TypeTest))
+#tts.addTests(TestLoader().loadTestsFromTestCase(ACLTest))
+#tts.addTests(TestLoader().loadTestsFromTestCase(ChangeEntryTest))
+
 #tts = TestLoader().loadTestsFromTestCase(QueryTest)
-#tts = TestLoader().loadTestsFromTestCase(FolderTest)
-#tts = TestLoader().loadTestsFromTestCase(TypeTest)
 
 #tts.addTest(CmisClientTest('testDefaultRepository'))
 #tts.addTest(QueryTest('testScore'))
@@ -42,9 +46,13 @@ tts = TestLoader().loadTestsFromTestCase(DocumentTest)
 #tts.addTest(DocumentTest('testCreateDocumentBinary'))
 #tts.addTest(DocumentTest('testSetContentStreamDoc'))
 #tts.addTest(DocumentTest('testSetContentStreamPWC'))
-#tts.addTest(DocumentTest('testUpdateProperties'))
+#tts.addTest(DocumentTest('testCheckin'))
+#tts.addTest(DocumentTest('testCheckinComment'))
+#tts.addTest(DocumentTest('testCheckinAfterGetPWC'))
+tts.addTest(DocumentTest('testUpdateProperties'))
 #tts.addTest(FolderTest('testGetChildren'))
 #tts.addTest(FolderTest('testGetDescendants'))
+#tts.addTest(FolderTest('testGetProperties'))
 #tts.addTest(FolderTest('testUpdateProperties'))
 #tts.addTest(DocumentTest('testCheckout'))
 #tts.addTest(DocumentTest('testCheckin'))
@@ -52,18 +60,13 @@ tts = TestLoader().loadTestsFromTestCase(DocumentTest)
 #tts.addTest(DocumentTest('testCancelCheckout'))
 #tts.addTest(DocumentTest('testAllowableActions'))
 #tts.addTest(RepositoryTest('testReturnVersion'))
+#tts.addTest(TypeTest('testTypeDefinition'))
+#tts.addTest(TypeTest('testTypeDescendants'))
 #tts.addTest(FolderTest('testGetTree'))
-
-result = TestResult()
-print "Running tests..."
-tts.run(result)
-if result.wasSuccessful():
-    print "No errors or failures!"
-else:
-    print "Errors (%d):" % len(result.errors)
-    for error in result.errors:
-        print error
-        
-    print "Failures (%d):" % len(result.failures)    
-    for failure in result.failures:
-        print failure
\ No newline at end of file
+#tts.addTest(ChangeEntryTest('testGetContentChanges'))
+#tts.addTest(ChangeEntryTest('testGetProperties'))
+#tts.addTest(ChangeEntryTest('testGetACL'))
+#tts.addTest(FolderTest('testPropertyFilter'))
+            
+ttr = TextTestRunner()
+ttr.run(tts)
\ No newline at end of file
